%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%            KS TESTING OF SIR MODEL                 %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/27/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

rng(123456); % Seed the random number generator for repeatable results

tic;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

k1    = 1.6633;
k2    = 0.44036;
Omega = 763;

x0 = [762;1];

tf = 6; % days

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve macroscopic equations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

y0 = x0/Omega;
[mu_time,mu] = ode23s(@(t,mu)rhsMacro(t,mu,k1,k2),[0;tf],y0);
LNA_mean = mu(end,:)*Omega;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve Lyapunov equations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[c_time,c] = ode23s(@(t,c)rhsLNA(t,c,mu_time,mu,k1,k2),mu_time,[0;0;0]);
LNA_cov = c(end,:)*Omega;
LNA_std = sqrt(LNA_cov(1:2));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform KS test on marginal distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Calculate sample size and significance level
alpha  = 0.01;
alpha0 = 1-sqrt(1-alpha); 

w0    = 0.1;
L = ceil(2*abs(log(alpha/2))/w0^2);

% Calculate critical values
s0Alpha  = sqrt(-.5*log(alpha/2))/(sqrt(L)+0.12+0.11/sqrt(L));
s0Alpha0 = sqrt(-.5*log(alpha0/2))/(sqrt(L)+0.12+0.11/sqrt(L));

XX1 = monteCarlo(k1,k2,Omega,x0,tf,L); % draw L Monte Carlo samples 

[H1,pValue1,KSstatistic1,criticalValue1] ...
                   = myKStest(XX1(:,1),LNA_mean(1),LNA_std(1),alpha0);
         
XX2 = monteCarlo(k1,k2,Omega,x0,tf,L); % draw L Monte Carlo samples 
                  
[H2, pValue2,KSstatistic2,criticalValue2] ...
                   = myKStest(XX2(:,2),LNA_mean(2),LNA_std(2),alpha0);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Combine p-values from marginal distribution using Tippett's method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

pValue = 1-(1-min(pValue1,pValue2))^2;

H = pValue < alpha; %1 for reject LNA; 0 for accept LNA
      
toc;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute exact solution of master equation using the IE method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

sir;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load exact results computed according to IE method of 
% Jenkinson & Goutsias (2012)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

load results/SIprobs_groundTruth

%
% Compute marginal PDFs
%

X1margExact = sum(SIprobs,2);
X2margExact = sum(SIprobs,1);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Kullback Liebler distance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

KL_dis = 0;

LNA_mean  = LNA_mean';
LNAcovMat = [LNA_cov(1) LNA_cov(3);LNA_cov(3) LNA_cov(2)];

invLNAcov = inv(LNAcovMat);

GaussianNorm = 1/sqrt(det(LNAcovMat)*(2*pi)^2);

for X1 = 0:1:762
    for X2 = 0:1:763
        if SIprobs(X1+1,X2+1)>0
            XminusMu = [X1;X2]-LNA_mean;
            % compute LNA prob
            LNAprob = GaussianNorm*exp(-XminusMu'*invLNAcov*XminusMu/2); %#ok<MINV>
            % compute KL divergence at this point
            KL_dis = KL_dis + log2(SIprobs(X1+1,X2+1)/LNAprob)*SIprobs(X1+1,X2+1);
        end
    end
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot decision bands
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

X      = 0:Omega-1;
LNAcdf = normcdf(X,LNA_mean(1),LNA_std(1));

[sampleCDF1,x1] = ecdf(XX1(:,1));

if min(x1)>0
    x1 = [0;x1];
    sampleCDF1 = [0;sampleCDF1];
end
if max(x1)<763
    x1 = [x1;762];
    sampleCDF1 = [sampleCDF1;1];
end

[sampleCDF2,x2] = ecdf(XX2(:,2));
if min(x2)>0
    x2 = [0;x2];
    sampleCDF2 = [0;sampleCDF2];
end
if max(x2)<763
    x2 = [x2;763];
    sampleCDF2 = [sampleCDF2;1];
end
sampleCDF2(1)=sampleCDF2(2);

figure
plot(X,LNAcdf,'LineWidth',1.25);
hold on
plot(X,min(LNAcdf+s0Alpha0,1),'r','LineWidth',1.25)
plot(X,max(LNAcdf-s0Alpha0,0),'r','LineWidth',1.25)
stairs(x1,sampleCDF1,'c','Linewidth',1.25)
hold off
xlabel('x_1');ylabel('Pr[ X_1(6) \leq x_1 ]')
title('{\bf DECISION BAND 1}')
axis([0 762 0 1.05])

saveas(gcf,'results/decisionBand_1.fig')

X      = 0:Omega;
LNAcdf = normcdf(X,LNA_mean(2),LNA_std(2));

figure
plot(X,LNAcdf,'LineWidth',1.25);
hold on
plot(X,min(LNAcdf+s0Alpha0,1),'r','LineWidth',1.25)
plot(X,max(LNAcdf-s0Alpha0,0),'r','LineWidth',1.25)
stairs(x2,sampleCDF2,'c','Linewidth',1.25)
hold off
xlabel('x_2');ylabel('Pr[ X_2(6) \leq x_2 ]')
title('{\bf DECISION BAND 2}')
axis([0 763 0 1.05])

saveas(gcf,'results/decisionBand_2.fig')

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot confidence bands
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

X = 0:(Omega-1);
LNAcdf1 = normcdf(X,LNA_mean(1),LNA_std(1));

figure
plot(X,LNAcdf1,'b','Linewidth',1.25);
hold on
stairs(X,cumsum(X1margExact),'k','Linewidth',1.25);
stairs(x1,min(sampleCDF1+s0Alpha,1),'g','Linewidth',1.25)
stairs(x1,max(sampleCDF1-s0Alpha,0),'g','Linewidth',1.25)
hold off
xlabel('x_1');
ylabel('Pr[ X_1(6) \leq x_1 ]')
title('{\bf CONFIDENCE BAND 1}')
axis([0 762 0 1.05])

saveas(gcf,'results/confidenceBand_1.fig')

X = 0:Omega;
LNAcdf2 = normcdf(X,LNA_mean(2),LNA_std(2));

figure
plot(X,LNAcdf2,'b','Linewidth',1.25);
hold on;
stairs(X,cumsum(X2margExact),'k','Linewidth',1.25);
stairs(x2,min(sampleCDF2+s0Alpha,1),'g','Linewidth',1.25)
stairs(x2,max(sampleCDF2-s0Alpha,0),'g','Linewidth',1.25)
xlabel('x_2');
ylabel('Pr[ X_2(6) \leq x_2 ]')
title('{\bf CONFIDENCE BAND 2}')
axis([0 763 0 1.05])

saveas(gcf,'results/confidenceBand_2.fig')