function [H, pValue, KSstatistic, criticalValue] = myKStest(x, LNAmean,LNAstd, alpha)
% Modified version of 'kstest' distributed under the following copyrights:
% Copyright 1993-2011 The MathWorks, Inc.
% $Revision: 1.1.8.5 $   $ Date: 1998/01/30 13:45:34 $

% References:
%   Massey, F.J., (1951) "The Kolmogorov-Smirnov Test for Goodness of Fit",
%         Journal of the American Statistical Association, 46(253):68-78.
%   Marsaglia, G., W.W. Tsang, and J. Wang (2003), "Evaluating Kolmogorov`s
%         Distribution", Journal of Statistical Software, vol. 8, issue 18.


L = length(x); % Sample size
[sampleCDF,x] = ecdf(x); % compute empiracle CDF

x = x(2:end);
 

nullCDF = normcdf(x,LNAmean,LNAstd);   % CDF has been specified at the observations in X.


%  2-sided test: T = max|S(x) - CDF(x)|.
delta1    =  sampleCDF(1:end-1) - nullCDF;   % Vertical difference at jumps approaching from the LEFT.
delta2    =  sampleCDF(2:end)   - nullCDF;   % Vertical difference at jumps approaching from the RIGHT.
deltaCDF  =  abs([delta1 ; delta2]);


KSstatistic   =  max(deltaCDF);


s = L*KSstatistic^2;

% For d values that are in the far tail of the distribution (i.e.
% p-values > .999), the following lines will speed up the computation
% significantly, and provide accuracy up to 7 digits.
if (s > 7.24) ||((s > 3.76) && (L > 99))
    pValue = 2*exp(-(2.000071+.331/sqrt(L)+1.409/L)*s);
else
    % Express d as d = (k-h)/n, where k is a +ve integer and 0 < h < 1.
    k = ceil(KSstatistic*L);
    h = k - KSstatistic*L;
    m = 2*k-1;

    % Create the H matrix, which describes the CDF, as described in Marsaglia,
    % et al. 
    if m > 1
        c = 1./gamma((1:m)' + 1);

        r = zeros(1,m);
        r(1) = 1; 
        r(2) = 1;

        T = toeplitz(c,r);

        T(:,1) = T(:,1) - (h.^(1:m)')./gamma((1:m)' + 1);

        T(m,:) = fliplr(T(:,1)');
        T(m,1) = (1 - 2*h^m + max(0,2*h-1)^m)/gamma(m+1);
     else
         T = (1 - 2*h^m + max(0,2*h-1)^m)/gamma(m+1);
     end

    % Scaling before raising the matrix to a power
    if ~isscalar(T)
        lmax = max(eig(T));
        T = (T./lmax)^L;
    else
        lmax = 1;
    end

    % Pr(Dn < d) = n!/n * tkk ,  where tkk is the kth element of Tn = T^n.
    % p-value = Pr(Dn > d) = 1-Pr(Dn < d)
    pValue = (1 - exp(gammaln(L+1) + L*log(lmax) - L*log(L)) * T(k,k));
end


% "H = 0" implies that we "Do not reject the null hypothesis at the
% significance level of alpha," and "H = 1" implies that we "Reject null
% hypothesis at significance level of alpha."
H  =  (pValue < alpha);


%
% Asymptotic Critical values
%
if L>35
    criticalValue = sqrt(-.5*log(alpha/2))/(sqrt(L)+0.12+0.11/sqrt(L));
else
    display('WARNING, THE KS test requires L>35 for asymptotic validity\n')
end

end