%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%         GAUSSIAN LEAPING IN SIR MODEL              %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 03/01/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

rng(123456); % Seed the random number generator for repeatable results

tic;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

k1    = 1.6633;
k2    = 0.44036;
Omega = 763;

x0 = [762;1];

tf = 6; % days

tau = 0.05; % days; tau leap parameter

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform TSKS test on marginal distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

alpha    = 0.01;
alpha0   = 1-(1-alpha)^(1/2); 

w0       = 0.1;
L        = ceil(4*abs(log(alpha/2))/w0^2);

s0Alpha  = sqrt(-.5*log(alpha/2))/(sqrt(L/2)+0.12+0.11/sqrt(L/2));

XX1exact = monteCarlo(k1,k2,Omega,x0,tf,L);    % draw L exact samples 
XX1tau   = GaussLeap(k1,k2,Omega,x0,tf,L,tau); % draw L approx samples
 
[h1,pValue1,ks2stat1] = kstest2(XX1exact(:,1),XX1tau(:,1),alpha0,'unequal');

XX2exact = monteCarlo(k1,k2,Omega,x0,tf,L);    % draw L exact samples 
XX2tau   = GaussLeap(k1,k2,Omega,x0,tf,L,tau); % draw L approx samples
 
[h2,pValue2,ks2stat2] = kstest2(XX2exact(:,2),XX2tau(:,2),alpha0,'unequal');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Combine p-values from marginal distributions using Tippett's method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

globalPvalue = 1-(1-min(pValue1,pValue2))^2;

H = globalPvalue < alpha; % 1 reject Gaussian leaping; 0 accept

toc;
      
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute exact solution of master equation using the IE method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

sir;

%
% Compute marginal PDFs
%

X1margExact = sum(SIprobs,2);
X2margExact = sum(SIprobs,1);
X1          = 0:(Omega-1);
X2          = 0:Omega;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot confidence bands
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[sampleCDF1exact,x1exact] = ecdf(XX1exact(:,1));

if min(x1exact)>0
    x1exact = [0;x1exact];
    sampleCDF1exact = [0;sampleCDF1exact];
end
if max(x1exact)<763
    x1exact = [x1exact;762];
    sampleCDF1exact = [sampleCDF1exact;1];
end

[sampleCDF1tau,x1tau] = ecdf(XX1tau(:,1));

if min(x1tau)>0
    x1tau = [0;x1tau];
    sampleCDF1tau = [0;sampleCDF1tau];
end
if max(x1tau)<763
    x1tau = [x1tau;762];
    sampleCDF1tau = [sampleCDF1tau;1];
end

if min(x1exact)>min(x1tau)
    x1exact = [min(x1tau);x1exact];
    sampleCDF1exact = [0;sampleCDF1exact];
end
if max(x1exact)<max(x1tau)
    x1exact = [x1exact;max(x1tau)];
    sampleCDF1exact = [sampleCDF1exact;1];
end

F1exact = cumsum(X1margExact);

if min(X1)>min(x1tau)
    X1 = [min(x1tau),X1];
    F1exact = [0;F1exact];
end
if max(X1)<max(x1tau)
    X1 = [X1,max(x1tau)];
    F1exact = [F1exact;1];
end

figure
stairs(x1tau,sampleCDF1tau,'b','Linewidth',1.25);
hold on;
stairs(X1,F1exact,'k','Linewidth',1.25);
stairs(x1exact,min(sampleCDF1exact+s0Alpha,1),'g','Linewidth',1.25)
stairs(x1exact,max(sampleCDF1exact-s0Alpha,0),'g','Linewidth',1.25)
hold off;
xlabel('x_1');
ylabel('Pr[ X_1(6) \leq x_1 ]')
title('{\bf CONFIDENCE BAND 1}')
axis([0 762 0 1.05])

saveas(gcf,'results/confidenceBand_1.fig')

[sampleCDF2exact,x2exact] = ecdf(XX2exact(:,2));

if min(x2exact)>0
    x2exact = [0;x2exact];
    sampleCDF2exact = [0;sampleCDF2exact];
end
if max(x2exact)<763
    x2exact = [x2exact;763];
    sampleCDF2exact = [sampleCDF2exact;1];
end

[sampleCDF2tau,x2tau] = ecdf(XX2tau(:,2));

if min(x2tau)>0
    x2exact = [0;x2tau];
    sampleCDF2tau = [0;sampleCDF2tau];
end
if max(x2tau)<763
    x2tau = [x2tau;762];
    sampleCDF2tau = [sampleCDF2tau;1];
end

if min(x2exact)>min(x2tau)
    x2exact = [min(x2tau);x2exact];
    sampleCDF2exact = [0;sampleCDF2exact];
end
if max(x2exact)<max(x2tau)
    x2exact = [x2exact;max(x2tau)];
    sampleCDF2exact = [sampleCDF2exact;1];
end

F2exact = cumsum(X2margExact);

if min(X2)>min(x2tau)
    X2 = [min(x2tau),X2];
    F2exact = [0,F2exact];
end
if max(X2)<max(x2tau)
    X2 = [X2,max(x2tau)];
    F2exact = [F2exact,1];
end

figure
stairs(x2tau,sampleCDF2tau,'b','Linewidth',1.25);
hold on;
stairs(X2,F2exact,'k','Linewidth',1.25);
stairs(x2exact,min(sampleCDF2exact+s0Alpha,1),'g','Linewidth',1.25)
stairs(x2exact,max(sampleCDF2exact-s0Alpha,0),'g','Linewidth',1.25)
hold off;
xlabel('x_2');
ylabel('Pr[ X_2(6) \leq x_2 ]')
title('{\bf CONFIDENCE BAND 2}')
axis([0 763 0 1.05])

saveas(gcf,'results/confidenceBand_2.fig')